import { LinkName } from './link-name';
import { PACKAGE_NAME } from './package-name';
import { createSerialOperation } from '@deep-foundation/deeplinks/imports/gql';
/**
  * Gets serial operations to insert {@link LinkName.PushNotification}
  *
  * @example
  * #### Insert {@link LinkName.PushNotification}
 ```ts
 const serialOperations = await getPushNotificationInsertSerialOperations({
   deep
 });
 await deep.serial({
   operations: serialOperations
 })
 ```
   * #### Insert {@link LinkName.PushNotification} with reserved link id
  ```ts
  const reservedLinkIds = await deep.reserve(2);
  const pushNotificationLinkId = reservedLinkIds.pop();
  const containLinkId = reservedLinkIds.pop();
  
  const serialOperations = await getPushNotificationInsertSerialOperations({
    deep,
    pushNotification: {
      title,
      body,
    },
    reservedLinkIds: {
      pushNotificationLinkId,
      containLinkId,
    }
  });
  await deep.serial({
    operations: serialOperations
  })
  ```
  */
export async function getPushNotificationInsertSerialOperations(param) {
    const { deep, pushNotification, containValue, containerLinkId, } = param;
    const { containLinkId, pushNotificationLinkId } = await getReservedLinkIds();
    const { containTypeLinkId, pushNotificationTypeLinkId } = await getTypeLinkIds();
    const serialOperations = [];
    const pushNotificationInsertSerialOperation = createSerialOperation({
        type: 'insert',
        table: 'links',
        objects: {
            id: pushNotificationLinkId,
            type_id: pushNotificationTypeLinkId,
        },
    });
    serialOperations.push(pushNotificationInsertSerialOperation);
    const valueOfPushNotificationInsertSerialOperation = createSerialOperation({
        type: 'insert',
        table: 'objects',
        objects: {
            link_id: pushNotificationLinkId,
            value: pushNotification,
        },
    });
    serialOperations.push(valueOfPushNotificationInsertSerialOperation);
    if (containerLinkId !== null) {
        const containInsertSerialOperation = createSerialOperation({
            type: 'insert',
            table: 'links',
            objects: {
                type_id: containTypeLinkId,
                from_id: containerLinkId || deep.linkId,
                to_id: pushNotificationLinkId,
            },
        });
        serialOperations.push(containInsertSerialOperation);
        const valueOfContainInsertSerialOperation = createSerialOperation({
            type: 'insert',
            table: 'strings',
            objects: {
                link_id: containLinkId,
                value: containValue,
            },
        });
        serialOperations.push(valueOfContainInsertSerialOperation);
    }
    return serialOperations;
    async function getReservedLinkIds() {
        let result = {
            containLinkId: 0,
            pushNotificationLinkId: 0,
        };
        const linksToReserveCount = Object.keys(result).length -
            Object.keys(param.reservedLinkIds || {}).length;
        const reservedLinkIds = linksToReserveCount > 0 ? await deep.reserve(linksToReserveCount) : [];
        result = {
            containLinkId: param.reservedLinkIds?.containLinkId ?? reservedLinkIds.pop(),
            pushNotificationLinkId: param.reservedLinkIds?.pushNotificationLinkId ?? reservedLinkIds.pop(),
        };
        return result;
    }
    async function getTypeLinkIds() {
        const result = {
            containTypeLinkId: param.typeLinkIds?.containTypeLinkId ||
                (await deep.id('@deep-foundation/core', 'Contain')),
            pushNotificationTypeLinkId: param.typeLinkIds?.pushNotificationTypeLinkId ||
                (await deep.id(PACKAGE_NAME, LinkName[LinkName.PushNotification])),
        };
        return result;
    }
}
