import { LinkName } from './link-name';
import { PACKAGE_NAME } from './package-name';
import { createSerialOperation } from '@deep-foundation/deeplinks/imports/gql';
/**
  * Gets serial operations to insert {@link LinkName.ServiceAccount}
  *
  * @example
  * #### Insert {@link LinkName.ServiceAccount}
 ```ts
 const serialOperations = await getServiceAccountInsertSerialOperations({
   deep
 });
 await deep.serial({
   operations: serialOperations
 })
 ```
   * #### Insert {@link LinkName.ServiceAccount} with reserved link id
  ```ts
  const reservedLinkIds = await deep.reserve(2);
  const serviceAccountLinkId = reservedLinkIds.pop();
  const containLinkId = reservedLinkIds.pop();
  
  const serialOperations = await getServiceAccountInsertSerialOperations({
    deep,
    serviceAccount: {
      title,
      body,
    },
    reservedLinkIds: {
      serviceAccountLinkId,
      containLinkId,
    }
  });
  await deep.serial({
    operations: serialOperations
  })
  ```
  */
export async function getServiceAccountInsertSerialOperations(param) {
    const { deep, serviceAccount, containValue, containerLinkId, shouldMakeActive = false } = param;
    const { containLinkId, serviceAccountLinkId, usesServiceAccountLinkId } = await getReservedLinkIds();
    const { containTypeLinkId, serviceAccountTypeLinkId, usesServiceAccountTypeLinkId } = await getTypeLinkIds();
    const serialOperations = [];
    const serviceAccountInsertSerialOperation = createSerialOperation({
        type: 'insert',
        table: 'links',
        objects: {
            id: serviceAccountLinkId,
            type_id: serviceAccountTypeLinkId,
        },
    });
    serialOperations.push(serviceAccountInsertSerialOperation);
    const valueOfServiceAccountInsertSerialOperation = createSerialOperation({
        type: 'insert',
        table: 'objects',
        objects: {
            link_id: serviceAccountLinkId,
            value: serviceAccount,
        },
    });
    serialOperations.push(valueOfServiceAccountInsertSerialOperation);
    if (containerLinkId !== null) {
        const containInsertSerialOperation = createSerialOperation({
            type: 'insert',
            table: 'links',
            objects: {
                id: containLinkId,
                type_id: containTypeLinkId,
                from_id: containerLinkId || deep.linkId,
                to_id: serviceAccountLinkId,
            },
        });
        serialOperations.push(containInsertSerialOperation);
        const valueOfContainInsertSerialOperation = createSerialOperation({
            type: 'insert',
            table: 'objects',
            objects: {
                link_id: containLinkId,
                value: containValue,
            },
        });
        serialOperations.push(valueOfContainInsertSerialOperation);
    }
    if (shouldMakeActive) {
        const usesServiceAccountInsertSerialOperation = createSerialOperation({
            type: 'insert',
            table: 'links',
            objects: {
                id: usesServiceAccountLinkId,
                type_id: usesServiceAccountTypeLinkId,
                from_id: deep.linkId,
                to_id: serviceAccountLinkId,
            },
        });
        serialOperations.push(usesServiceAccountInsertSerialOperation);
    }
    return serialOperations;
    async function getReservedLinkIds() {
        let result = {
            containLinkId: 0,
            serviceAccountLinkId: 0,
            usesServiceAccountLinkId: 0
        };
        const linksToReserveCount = Object.keys(result).length -
            Object.keys(param.reservedLinkIds || {}).length;
        const reservedLinkIds = linksToReserveCount > 0 ? await deep.reserve(linksToReserveCount) : [];
        result = {
            containLinkId: param.reservedLinkIds?.containLinkId ?? reservedLinkIds.pop(),
            serviceAccountLinkId: param.reservedLinkIds?.serviceAccountLinkId ?? reservedLinkIds.pop(),
            usesServiceAccountLinkId: param.reservedLinkIds?.usesServiceAccountLinkId ?? reservedLinkIds.pop(),
        };
        return result;
    }
    async function getTypeLinkIds() {
        const result = {
            containTypeLinkId: param.typeLinkIds?.containTypeLinkId ||
                (await deep.id('@deep-foundation/core', 'Contain')),
            serviceAccountTypeLinkId: param.typeLinkIds?.serviceAccountTypeLinkId ||
                (await deep.id(PACKAGE_NAME, LinkName[LinkName.ServiceAccount])),
            usesServiceAccountTypeLinkId: param.typeLinkIds?.usesServiceAccountTypeLinkId ||
                (await deep.id(PACKAGE_NAME, LinkName[LinkName.UsesServiceAccount])),
        };
        return result;
    }
}
